///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2005 Xilinx, Inc.
// All Rights Reserved
///////////////////////////////////////////////////////////////////////////////
// ____ ____
// / /\/ /
// /___/ \ / Vendor: Xilinx
// \ \ \/ Version  : 1.5
// \ \ Application : MIG
// / / Filename    : controller.v
// /___/ /\ 
// \ \ / \
// \___\/\___\
// Sarala
//
//Design Name : controller
//Description :  Main DDR SDRAM controller block. This includes the following
//                features:
//                - The controller state machine that controls the 
//                initialization process upon power up, as well as the 
//                read, write, and refresh commands. 
//                - Accepts and decodes the user commands.
//                - Generates the address and Bank address signals
//                - Generates control signals for other modules, including
//                the control signals for the dqs_en block.
///////////////////////////////////////////////////////////////////////////////

`include "D:\\sajat\\xst_vlog_bl2cl2_parameters_0.v"


`timescale 1ns/100ps

module xst_vlog_bl2cl2_controller_0(
                clk,
		  // clk180,	
		 rst0,
	         rst180,
	         address,
	         bank_address,
	         config_register,
	         command_register,
	         burst_done,
		 ddr_rasb_cntrl,
	         ddr_casb_cntrl,
	         ddr_web_cntrl,
	         ddr_ba_cntrl,
	         ddr_address_cntrl,
	         ddr_cke_cntrl,
	         ddr_csb_cntrl,
	         dqs_enable,
	         dqs_reset,
	         write_enable,
	         rst_calib,
	         rst_dqs_div_int,
	         cmd_ack,
	         init,
	         ar_done,
                 wait_200us,
                 auto_ref_req
                 );

 
   input          clk;  
  //input          clk180;                      
   input          rst0;            
   input 	        rst180;          
// input[((`row_address + `column_address)-1):0] 	  address;         
   input[((`row_address + `col_ap_width)  -1):0] 	  address;         

   input[`bank_address-1:0] 	  bank_address;    
   input[9:0] 	  config_register; 
   input[2:0] 	  command_register;
   input          burst_done;
  
   output         ddr_rasb_cntrl;    
   output         ddr_casb_cntrl;    
   output         ddr_web_cntrl;    
   output[`bank_address-1:0]    ddr_ba_cntrl;     
   output[`row_address-1:0]   ddr_address_cntrl;
   output         ddr_cke_cntrl; 
   output         ddr_csb_cntrl;     
   output         dqs_enable;  
   output         dqs_reset;  
   output         write_enable;
   output         rst_calib;   
   output         rst_dqs_div_int;
   output         cmd_ack;
   output         init;  
   output         ar_done;

   input          wait_200us ;
   output         auto_ref_req; 
   
   reg  [`row_address-1:0]ddr_address_cntrl;
   reg  [`bank_address-1:0]ddr_ba_cntrl;   
   
parameter [3:0] IDLE = 0,
                PRECHARGE = 1,
                LOAD_MODE_REG = 2,
                AUTO_REFRESH =3,
                ACTIVE = 4,
                FIRST_WRITE =5,
                WRITE_WAIT = 6,
                BURST_WRITE = 7,
                READ_AFTER_WRITE = 8,
                PRECHARGE_AFTER_WRITE = 9,
                PRECHARGE_AFTER_WRITE_2 = 10,
                READ_WAIT =11,
                BURST_READ = 12,
                BURST_STOP = 13,//D
                ACTIVE_WAIT = 14;//E

reg          ar_done;
reg          write_enable;
//reg [12:0]   ddr_address;
//reg [1:0]    ddr_ba;

reg [3:0]    next_state;
reg [3:0]    next_state1;

wire         ack_reg;
wire         ack_o;
//reg [((`row_address + `column_address)-1):0]   address_reg;
  reg [((`row_address + `col_ap_width)  -1):0]   address_reg;

wire [`row_address-1:0]  address_config;
reg          auto_ref;
wire         auto_ref1;
wire         AUTOREF_value;
reg          AUTO_REF_detect;
reg          AUTO_REF_detect1;
reg          AUTO_REF_pulse_end;
reg [10:0]   AUTOREF_COUNT;
wire [10:0]  AUTOREF_CNT_val;
reg          Auto_Ref_issued;
wire         Auto_Ref_issued_p;
reg [5:0]   RFC_COUNTER_value;

//wire          auto_refresh_cmd;
wire         AR_done_p;
//wire          Burst_terminate;
reg [`bank_address-1:0]    BA_address_active;
reg          BA_address_conflict;
reg [`bank_address-1:0]    BA_address_reg;
reg [2:0]    burst_length;
wire [2:0]   burst_cnt_max;
//reg [1:0]    CAS_COUNT;
  reg [2:0]    CAS_COUNT;  //Modifiedd by Abhishake for BL=8
//wire [1:0]   cas_count_value;
  wire [2:0]   cas_count_value; //Modifiedd by Abhishake for BL=8

reg [2:0]    cas_latency;
/* reg [`column_address-1:0]    column_address_reg;
reg [`column_address-1:0]    column_address_reg1;
reg [`column_address-1:0]    column_address_reg2;
reg [`column_address-1:0]    column_address_reg3;
reg [`column_address-1:0]    column_address_reg4;
reg [`column_address-1:0]    column_address_reg5;
reg [`column_address-1:0]    column_address_reg6;
wire[`column_address-1:0]    column_address;
*/
reg [`row_address -1:0]    column_address_reg;
reg [`row_address -1:0]    column_address_reg1;
reg [`row_address -1:0]    column_address_reg2;
reg [`row_address -1:0]    column_address_reg3;
reg [`row_address -1:0]    column_address_reg4;
reg [`row_address -1:0]    column_address_reg5;
reg [`row_address -1:0]    column_address_reg6;
wire[`row_address -1:0]    column_address;

reg [2:0]    command_reg;
reg [9:0]    config_reg;
reg          CONFLICT;
wire         CONFLICT_value;
wire         ddr_rasb1;
wire         ddr_casb1;
wire         ddr_web1;
reg          ddr_rasb2;
reg          ddr_casb2;
reg          ddr_web2;
reg          ddr_rasb3;
reg          ddr_casb3;
reg          ddr_web3;
reg          ddr_rasb4;
reg          ddr_casb4;
reg          ddr_web4;
reg          ddr_rst_dqs_rasb4;
reg          ddr_rst_dqs_casb4;
reg          ddr_rst_dqs_web4;
reg          ddr_rasb5;
reg          ddr_casb5;
reg          ddr_web5;
wire[`bank_address-1:0]  ddr_ba1;
reg [`bank_address-1:0]  ddr_ba2;
reg [`bank_address-1:0]  ddr_ba3;
reg [`bank_address-1:0]  ddr_ba4;
reg [`bank_address-1:0]  ddr_ba5;
wire [`row_address-1:0]  ddr_address1;
reg [`row_address-1:0]   ddr_address2;
reg [`row_address-1:0]   ddr_address3;
reg [`row_address-1:0]   ddr_address4;
reg [`row_address-1:0]   ddr_address5;
wire         DQS_enable_out;
wire         DQS_reset_out;
wire [2:0]   INIT_COUNT_value;
reg [2:0]    INIT_COUNT;
wire [7:0]   DLL_RST_COUNT_value;
reg [7:0]    DLL_RST_COUNT;
reg          INIT_DONE;
wire         init_done_value;
reg          init_memory;
wire         init_mem;
wire          initialize_memory;
wire          ld_mode;
wire [1:0]   MRD_COUNT_value;
reg [10:0]  max_ref_cnt;
reg [1:0]     MRD_COUNT;
wire          PRECHARGE_CMD;
wire [3:0]   ras_count_value;
reg [3:0]    RAS_COUNT;
wire         rdburst_chk;
wire          read_cmd;
reg          read_cmd1;
reg          read_cmd2;
reg          read_cmd3;
reg          read_cmd4;
reg          read_cmd5;
reg          read_cmd6;
reg          read_cmd7;
reg          read_cmd8;
reg          read_rcd_end;
reg          read_cmd_reg;
wire         read_write_state;
reg [1:0]    RRD_COUNT;
reg [2:0]    RCDR_COUNT;
reg [1:0]    RCDW_COUNT;
wire [2:0]   rp_cnt_value;
wire [4:0]   RFC_COUNT_value;
reg          RFC_COUNT_reg;  
reg          AR_Done_reg;  
wire [1:0]   RRD_COUNT_value;
wire [2:0]   RCDR_COUNT_value;
wire [1:0]   RCDW_COUNT_value;
wire [3:0]   RC_COUNT_value;
wire [2:0]   rdburst_end_cnt_value;
reg [2:0]    RDBURST_END_CNT;
reg          rdburst_end_1;
reg          rdburst_end_2;
reg          rdburst_end_3;
reg          rdburst_end_4;
reg          rdburst_end_5;
reg          rdburst_end_6;
reg          rdburst_end_7;
reg          rdburst_end_8;
wire         rdburst_end_r;
wire         read_enable_out_r;
wire         rdburst_end;
reg [2:0]    RP_COUNT;
reg [3:0]    RC_COUNT;
reg [4:0]    RFC_COUNT;
wire         read_enable_out;
wire [`row_address-1:0]   ROW_ADDRESS;
reg  [`row_address-1:0]   row_address_reg;
reg  [`row_address-1:0]   row_address_active_reg;
reg          row_address_conflict;
reg          rst_dqs_div_r;

//reg		 rst_dqs_div_r11;
//reg		 rst_dqs_div_r2;

wire		 rst_dqs_div_r1;
reg          dly_dqs_div_r;  //Added to delay the rst_dqs_div_r by 1 clock pulse for BL = 2 .

wire [2:0]   wrburst_end_cnt_value;
reg  [2:0]   wrburst_end_cnt;
wire         wrburst_end;
reg          wrburst_end_1;
reg          wrburst_end_2;
reg          wrburst_end_3;
reg          wrburst_end_4;
reg          wrburst_end_5;
reg          wrburst_end_6;
reg          wrburst_end_7;
reg          wrburst_end_8;
reg          wrburst_end_9;
wire         wrburst_chk;
reg  [1:0]    WR_COUNT;
wire [1:0]   WR_COUNT_value;
wire         write_enable_out;
//reg          write_enable_int; //Added for Registered Dimms

reg          write_cmd;
wire          write_cmd_in;
reg          write_cmd2;
reg          write_cmd3;
reg          write_cmd4;
reg          write_cmd5;
reg          write_cmd1;
reg          write_cmd6;
reg          write_cmd7;
reg          write_cmd8;
wire         GND;
reg [2:0]    dqs_div_cascount;
reg [2:0]    dqs_div_rdburstcount;
wire         rst_dqs_div_int;
reg          DQS_enable1;
reg          DQS_enable2;
reg          DQS_enable3;
reg          DQS_enable4;
reg          DQS_reset1_clk0;
reg          DQS_reset2_clk0;
reg          DQS_reset3_clk0;
reg          DQS_reset4_clk0;
reg          DQS_enable_int;
reg          DQS_reset_int;
reg          rst180_r;
reg          rst0_r;
wire         GO_TO_ACTIVE_value;
reg          GO_TO_ACTIVE;


 reg      rpCnt0;
 reg      rpCnt1;

 reg     mrdCnt0;          
 reg     mrdCnt1;          
 
 reg     rcdrCnt0;       
 reg     rcdrCnt1;        
 
 reg     rcdwCnt0;      
 reg     rcdwCnt1;  

 reg     rcCnt0;     

wire  accept_cmd_in;

reg ldMdReg_flag ;
reg precharge_flag;
reg aref_flag;
reg idle_flag;

 reg  auto_ref_wait;
 reg  auto_ref_wait1;
 reg  auto_ref_wait2;


//  Input : CONFIG REGISTER FORMAT 
// config_register = {   EMR(Enable/Disable DLL),
//                       BMR (Normal operation/Normal Operation with Reset DLL),
//                       BMR/EMR,
//                       CAS_latency (3),
//                       Burst type ,
//                       Burst_length (3) }
//
// Input : COMMAND REGISTER FORMAT
//          000  - NOP
//          001  - Precharge 
//          010  - Auto Refresh
//          011  - SElf REfresh
//          100  - Write Request
//          101  - Load Mode Register
//          110  - Read request
//          111  - Burst terminate
//
// Input : Address format
//   row address = input address(19 downto 8)
//   column addrs = input address( 7 downto 0)
//


assign ddr_csb_cntrl = 1'b0; // dip3;
assign ddr_cke_cntrl = ~wait_200us;

 


//assign ROW_ADDRESS = address_reg[((`row_address + `column_address)-1):`column_address]; 
  assign ROW_ADDRESS = address_reg[((`row_address + `col_ap_width  )-1):`col_ap_width]; 

//assign column_address = address_reg[`column_address-1:0];
  assign column_address = address_reg[`row_address  -1:0];
assign init = INIT_DONE;
assign GND = 1'b0;


assign ddr_rasb_cntrl = ddr_rasb2;
assign ddr_casb_cntrl = ddr_casb2;
assign ddr_web_cntrl = ddr_web2;

assign auto_ref_req = auto_ref_wait;

always @ (negedge clk)
//always @ (posedge clk180)
begin
  rst180_r <= rst180;
end

always @ (posedge clk)
begin
  rst0_r <= rst0;
end

//********************************************************************************************
// register input commands from the user
// 
//********************************************************************************************

  always @ (negedge clk)       //(posedge clk180)
  begin
    if (rst180_r == 1'b1)
      begin
        config_reg <= 10'b0000000000;
        command_reg <= 3'b000;
        row_address_reg <= `row_address'b0;
//        column_address_reg <= `column_address'b0;
          column_address_reg <= `row_address'b0;
        BA_address_reg <= `bank_address'b0;
//        address_reg <= `row_address + `column_address'b0;
          address_reg <= `row_address + `col_ap_width'b0;

      end
    else
      begin
        config_reg <= config_register;
        command_reg <= command_register;
        row_address_reg <= ROW_ADDRESS;
        column_address_reg <= column_address;
        BA_address_reg <= bank_address;
        address_reg <= address;
      end
  end
  
always @ (negedge clk)       //(posedge clk180)
begin
  if (rst180_r == 1'b1)
    begin
     burst_length <= 3'b000;
     cas_latency  <= 3'b000;
    end
  else
    begin
     burst_length <= config_reg[2:0];    
     cas_latency  <= config_reg[6:4];
    end
end



//assign accept_cmd_in = ((next_state1 == IDLE) && (RP_COUNT == 3'b000) && RFC_COUNT_reg == 1'b1) ;

//assign accept_cmd_in = ((next_state1 == IDLE || idle_flag) && (rpCnt0 == 1'b1) && RFC_COUNT_reg == 1'b1) ;

//assign accept_cmd_in = ((next_state1 == IDLE || idle_flag) && (rpCnt0 == 1'b1) && RFC_COUNT_reg == 1'b1 && !auto_ref_wait);

assign accept_cmd_in = ((next_state == IDLE ) && rpCnt0 && RFC_COUNT_reg  && !auto_ref_wait);


assign PRECHARGE_CMD     = (command_register == 3'b001 && accept_cmd_in == 1'b1);
assign initialize_memory = (command_register == 3'b010);
assign write_cmd_in      = (command_register == 3'b100 && accept_cmd_in == 1'b1);
assign ld_mode           = (command_register == 3'b101 && accept_cmd_in == 1'b1);
assign read_cmd          = (command_register == 3'b110 && accept_cmd_in == 1'b1);


//**************************************************************************
// write_cmd is used to determine when there is a new write request
//**************************************************************************

// register write_cmd until WRITE command needs to be issued

  always @ (negedge clk)       //(posedge clk180)
  begin
     if (rst180_r == 1'b1)
      begin
       write_cmd1  <= 1'b0;
       write_cmd2  <= 1'b0;
       write_cmd3  <= 1'b0;
       write_cmd4  <= 1'b0;
       write_cmd5  <= 1'b0;
       write_cmd6  <= 1'b0;
       write_cmd7  <= 1'b0;
       write_cmd8  <= 1'b0;
       write_cmd   <= 1'b0;
      end
     else
      begin
	  if (accept_cmd_in)
           write_cmd1 <= write_cmd_in;
       write_cmd2 <= write_cmd1;
       write_cmd3 <= write_cmd2;
       write_cmd4 <= write_cmd3;
       write_cmd5 <= write_cmd4; 
       write_cmd6 <= write_cmd5; 
       write_cmd7 <= write_cmd6; 
       write_cmd8  <= write_cmd7;   
       write_cmd  <= write_cmd6;
      end               
  end
  
//************************************************************************
// register read cmd until READ command needs to be issued 
//************************************************************************

  always @ (negedge clk)       //(posedge clk180)
  begin
     if (rst180_r == 1'b1)
      begin
       read_cmd1      <= 1'b0;
       read_cmd2      <= 1'b0;
       read_cmd3      <= 1'b0;
       read_cmd4      <= 1'b0;
       read_cmd5      <= 1'b0;
       read_cmd6      <= 1'b0;
       read_cmd7      <= 1'b0;
       read_cmd8      <= 1'b0;
       read_cmd_reg   <= 1'b0;
       read_rcd_end   <= 1'b0;
      end
     else
      begin
	  if (accept_cmd_in)
       read_cmd1       <= read_cmd;
       read_cmd2       <= read_cmd1;
       read_cmd3       <= read_cmd2;
       read_cmd4       <= read_cmd3;
       read_cmd5       <= read_cmd4; 
       read_cmd6       <= read_cmd5;  
       read_cmd7       <= read_cmd6;
       read_cmd_reg    <= read_cmd7; 
       read_cmd8       <= read_cmd7; 
       read_rcd_end    <= read_cmd8;
      end
  end
  
//********************************************************************************************                
// MRD Counter
// an executable command can be issued only after Tmrd(2 cycles) after a LMR command is issued
//********************************************************************************************
assign MRD_COUNT_value = (ldMdReg_flag) ? 2'b11 :


                         // (next_state1 == LOAD_MODE_REG) ? 2'b11 :
                         // (MRD_COUNT != 2'b00) ? (MRD_COUNT - 1'b1) :
                            (mrdCnt0 != 1'b1) ? (MRD_COUNT - 1'b1) :
                           2'b00;
                          
//********************************************************************************************                
// RFC Counter
// an executable command can be issued only after Trfc(60 ns => 60/5 = 12 cycles) 
//                                after a AUTOREFRESH command is issued
//********************************************************************************************

assign RFC_COUNT_value = (aref_flag) ?  (RFC_COUNTER_value):

                     //(next_state1 == AUTO_REFRESH) ?  (RFC_COUNTER_value):
			  (RFC_COUNT != 5'b00000) ?  (RFC_COUNT - 1'b1):
			   5'b00000;

//********************************************************************************************                
// RP Counter
// an executable command can be issued only after Trp(20 ns for a -5 device => 4 cycles) 
//                        after a PRECHARGE command is issued
//********************************************************************************************
assign rp_cnt_value = (precharge_flag) ? 3'b100 :

                      //(next_state1 == PRECHARGE) ? 3'b100 :
                     
                      // (RP_COUNT != 3'b000) ? (RP_COUNT - 1'b1) :
                        (rpCnt0 != 1'b1) ? (RP_COUNT - 1'b1) : 
                       3'b000;
                       
//********************************************************************************************                
// RRD Counter
// minimum interval between successive ACTIVE commands to different banks - Trrd
// 2 clock cycles
//********************************************************************************************
assign RRD_COUNT_value = (next_state1 == ACTIVE) ? 2'b10 :
                          (RRD_COUNT != 2'b00) ? (RRD_COUNT - 1'b1) :
                           2'b00;
                           
//*********************************************************************************************
// ACTIVE to READ/WRITE counter
// RCDr counter
// ACTIVE to READ delay - (-5) device requires 20 ns of delay => 4 clock cycles
//
// RCDW counter
// ACTIVE to WRITE delay - (-5) device requires 10 ns of delay => 2 clock cycles
//
//*********************************************************************************************


assign RCDR_COUNT_value =  (next_state1 == ACTIVE) ? 3'b100 :
                           // (RCDR_COUNT != 3'b000) ? (RCDR_COUNT - 1'b1) :
                           (rcdrCnt0 != 1'b1) ? (RCDR_COUNT - 1'b1) :
                            3'b000;

assign RCDW_COUNT_value = (next_state1 == ACTIVE) ? 2'b11 :
                          //(RCDW_COUNT != 2'b00) ? (RCDW_COUNT - 1'b1) :
                        
                          (rcdwCnt0 != 1'b1) ? (RCDW_COUNT - 1'b1) :
                       
                           2'b00;  
                           
//*********************************************************************************************
// ACTIVE to PRECHARGE counter
// RAS counter
// ACTIVE to PRECHARGE delay -
// the memory device require 40 ns (8 clock cycles)after issuing an ACTIVE command before issuing a
// PRECHARGE command
//
//*********************************************************************************************
assign ras_count_value = (next_state1 == ACTIVE) ? 4'b1000 :
                          (RAS_COUNT != 4'b0000) ? (RAS_COUNT - 1'b1) :
                          4'b0000;

//**********************************************************************************************
// RC counter
// an ACTIVE command to a different row can be issued only after the previous row has been 
// precharged & after Trc is met 
// Trc = 60 ns = 12 clock cycles
//**********************************************************************************************
assign RC_COUNT_value = (next_state1 == ACTIVE) ? 4'b1100 :
                        // (RC_COUNT != 4'b0000) ? (RC_COUNT - 1'b1) :

                         (rcCnt0 != 1'b1) ? (RC_COUNT - 1'b1) :
                         4'b0000;


//********************************************************************************************                
// WR Counter
// a PRECHARGE command can be applied only after 2 cycles after a WRITE command has finished 
// executing
//********************************************************************************************
assign WR_COUNT_value = (next_state1 == PRECHARGE_AFTER_WRITE) ? 2'b11 :
                         (WR_COUNT != 2'b00) ? (WR_COUNT - 1'b1) :
                         2'b00;

//********************************************************************************************************
// Auto refresh counter - the design uses AUTO REFRESH
// 
// the DDR SDRAM requires AUTO REFRESH cycles at an average interval of 7.8 us
// Hence, a counter value to obtain a 7.8.6 us clock for Auto Refresh
// Refresh Request is raised for every 7.7 us
// Hence for 200MHz frequency,The Refresh_count_value = freq * refresh_time_period = 200*7.7 = 1540 
//**********************************************************************************************************


assign AUTOREF_value   = (AUTOREF_COUNT == max_ref_cnt)  ? 1'b1 : 1'b0;
                  
//assign AUTOREF_CNT_val = ((AUTOREF_COUNT == max_ref_cnt) || !INIT_DONE )? 11'b0 :
//                          AUTOREF_COUNT + 1'b1;



assign AUTOREF_CNT_val = ((AUTOREF_value) || !INIT_DONE )? 11'b0 : //sarala, 22nd june
                          AUTOREF_COUNT + 1'b1;



always @ (negedge clk)       //(posedge clk180)   
begin
  if (rst180_r == 1'b1)
  begin
     AUTOREF_COUNT     <= 11'b00000000000;
     RFC_COUNTER_value <= `rfc_count_value;
     max_ref_cnt       <=  `max_ref_cnt;
  end 
  else
     begin
     AUTOREF_COUNT     <= AUTOREF_CNT_val;
     RFC_COUNTER_value <= `rfc_count_value;
     max_ref_cnt       <= `max_ref_cnt;
     end 
end

always @ (negedge clk)       //(posedge clk180)
begin
  if (rst180_r == 1'b1)
    begin
      AUTO_REF_pulse_end <= 1'b0;
      AUTO_REF_detect1   <= 1'b0;
      AUTO_REF_detect    <= 1'b0; 
    end
  else
    begin
      AUTO_REF_detect1   <= AUTOREF_value;
      AUTO_REF_detect    <= AUTO_REF_detect1;
      AUTO_REF_pulse_end <= AUTO_REF_detect;
    end
end
    
//assign auto_ref1 = ((AUTO_REF_detect == 1'b1) && (AUTO_REF_pulse_end != 1'b1)) ? 1'b1 : 1'b0;


assign auto_ref1 = ((AUTO_REF_detect == 1'b1) && (AUTO_REF_pulse_end != 1'b1)) ; // Sarala, June 22
           
//assign AR_done_p = (AR_Done_reg == 1'b1) ? 1'b1 : 1'b0;  

assign AR_done_p = (AR_Done_reg == 1'b1); // Sarala June 22 
   
           
always @ (negedge clk)       //(posedge clk180)
begin
  if (rst180_r == 1'b1)
    begin
     auto_ref_wait <= 1'b0;
     ar_done  <= 1'b0;
     Auto_Ref_issued <= 1'b0;
    end
  else
    begin
     if (auto_ref1 && !auto_ref_wait) 
        auto_ref_wait <= 1'b1;
     else if (Auto_Ref_issued) 
        auto_ref_wait <= 1'b0;
     else  
        auto_ref_wait <= auto_ref_wait;
        
     ar_done         <= AR_done_p;
     Auto_Ref_issued <= Auto_Ref_issued_p;

    end
end      

always @ (negedge clk)       //(posedge clk180)
begin
  if (rst180_r == 1'b1)
  begin
     auto_ref_wait1 <= 1'b0;
     auto_ref_wait2 <= 1'b0;
     auto_ref       <= 1'b0;
  end
  else
  begin
     if (Auto_Ref_issued)
     begin
        auto_ref_wait1 <= 1'b0;
        auto_ref_wait2 <= 1'b0;
        auto_ref       <= 1'b0;
     end
     else
     begin
        auto_ref_wait1  <= auto_ref_wait;
        auto_ref_wait2  <= auto_ref_wait1;
        auto_ref        <= auto_ref_wait2;
     end
  end
end      

//assign Auto_Ref_issued_p = (next_state1 == AUTO_REFRESH) ? 1'b1 : 1'b0;


assign Auto_Ref_issued_p = (next_state1 == AUTO_REFRESH || aref_flag); 

//**********************************************************************************************
// Burst count value counter when there are cosecutive READs or WRITEs
// While doing consecutive READs or WRITEs, the Burst_count value determines when the next 
// READ or WRITE command should be issued. The burst length is determined while loading the 
// Load Mode Register
// burst_cnt_max shows the number of clock cycles for each burst
// burst_cnt_max = 1 for a burst length of 2, since it is ddr
//**********************************************************************************************
assign burst_cnt_max = (burst_length == 3'b001) ? 3'b001 :
                       (burst_length == 3'b010) ? 3'b010 :
                       (burst_length == 3'b011) ? 3'b100 :
                       (burst_length == 3'b111) ? 3'b100 :
                       3'b000;

//********************************************************************************************
// CAS latency counter
// CAS latencies of 2,3,4 can be set using Mode register bits M(6:4)
//
//      M6  M5  M4   CAS latency
//      0    1   0 -    2
//      0    1   1 -    3
//      1    0   0 -    4
//      others     -   reserved
// This design uses a CAS latency of 3 for a clock rate of 200 MHz
//
//********************************************************************************************
//assign cas_count_value = (next_state1 == BURST_READ) ? 2'b10 :
  assign cas_count_value = (next_state1 == BURST_READ) ? burst_cnt_max :  //Modified by Abhishake for BL=8
                         (CAS_COUNT != 3'b000) ? (CAS_COUNT - 1'b1) :
                         CAS_COUNT;



//assign rdburst_end_cnt_value = (CAS_COUNT == 2'b01) ? burst_cnt_max :  
assign rdburst_end_cnt_value = (CAS_COUNT == 3'b001) ? burst_cnt_max :  //Modified by Abhishake for BL=8
                               (RDBURST_END_CNT != 3'b000) ? (RDBURST_END_CNT - 1'b1) :
                               3'b000;


assign wrburst_end_cnt_value = ((next_state1 == FIRST_WRITE) || (next_state1 == BURST_WRITE)) ? burst_cnt_max :
                               (wrburst_end_cnt != 3'b000) ? (wrburst_end_cnt - 1'b1) :
                               3'b000;


assign wrburst_chk = ((next_state1 == BURST_WRITE) || (next_state1 == WRITE_WAIT)) ? 1'b1 : 1'b0;
                      
assign rdburst_chk  = ((next_state1 == BURST_READ) || (next_state1 == READ_WAIT)) ? 1'b1 : 1'b0;

always @ (negedge clk)       //(posedge clk180)
begin
  if (rst180_r == 1'b1)
    begin
      rdburst_end_1 <= 1'b0;
      rdburst_end_2 <= 1'b0;
      rdburst_end_3 <= 1'b0;
      rdburst_end_4 <= 1'b0;
      rdburst_end_5 <= 1'b0; 
    end
  else
    begin
      rdburst_end_2 <= rdburst_end_1;
      rdburst_end_3 <= rdburst_end_2;
      rdburst_end_4 <= rdburst_end_3;
      rdburst_end_5 <= rdburst_end_4;
      rdburst_end_6 <= rdburst_end_5;
      rdburst_end_7 <= rdburst_end_6;
      rdburst_end_8 <= rdburst_end_7;
//      if (((CAS_COUNT[1] == 1'b1) || (CAS_COUNT[0] == 1'b1) || (burst_cnt_max == 3'b010)) && (burst_done == 1'b1))
      if (burst_done == 1'b1) //BL=8
        rdburst_end_1 <= 1'b1;
      else
        rdburst_end_1 <= 1'b0;
    end
end

assign rdburst_end_r = rdburst_end_3 || rdburst_end_4 || rdburst_end_5 || rdburst_end_6 || rdburst_end_7 || rdburst_end_8;
assign rdburst_end = rdburst_end_1 || rdburst_end_2 ;

always @ (negedge clk)       //(posedge clk180)
begin
  if (rst180_r == 1'b1)
    begin
     wrburst_end_1 <= 1'b0;
     wrburst_end_2 <= 1'b0;
     wrburst_end_3 <= 1'b0;
     wrburst_end_4 <= 1'b0;
     wrburst_end_5 <= 1'b0;
     wrburst_end_6 <= 1'b0;
     wrburst_end_7 <= 1'b0;
     wrburst_end_8 <= 1'b0;
     wrburst_end_9 <= 1'b0;
    end
  else
    begin
     wrburst_end_2  <= wrburst_end_1;  
     wrburst_end_3  <= wrburst_end_2;
     wrburst_end_4  <= wrburst_end_3; 
     wrburst_end_5  <= wrburst_end_4; 
     wrburst_end_6  <= wrburst_end_5; 
     wrburst_end_7  <= wrburst_end_6; 
     wrburst_end_8  <= wrburst_end_7; 
   //  if (((wrburst_end_cnt[1] == 1'b1) || (wrburst_end_cnt[0] == 1'b1) || (burst_cnt_max == 3'b010)) && (burst_done == 1'b1))
     if  (burst_done == 1'b1)   //BL =8
         wrburst_end_1 <= 1'b1;
     else 
         wrburst_end_1 <= 1'b0;
    end
end

assign wrburst_end = wrburst_end_1 || wrburst_end_2 || wrburst_end_3;

//**********************************************************************************************
// to generate the Data Strobe enable and reset signal
// The DQS signal needs to be generated center aligned with the data. 
// The controller generates the DQS enable signal when the state machine is in the FIRST_WRITE
// state,to take care of the write preamble
//**********************************************************************************************

assign DQS_enable_out = ((next_state1 == FIRST_WRITE) || (next_state1 == BURST_WRITE)  || (wrburst_end_cnt != 3'b000)) ? 1'b1 : 1'b0;

assign DQS_reset_out  = (next_state1 == FIRST_WRITE) ? 1'b1 : 1'b0;

//assign dqs_enable = DQS_enable2;
//assign dqs_reset  = DQS_reset2_clk0;


  assign dqs_enable = DQS_enable2;
  assign dqs_reset  = DQS_reset2_clk0; 

  

//assign dqs_enable = DQS_enable3;		 // Added for Registered Dimms                          
//assign dqs_reset  = DQS_reset3_clk0;         // Added for Registered Dimms  

always @ (negedge clk)       //(posedge clk180)
begin
  if (rst180_r == 1'b1)
    begin
      DQS_enable_int <= 1'b0;
      DQS_reset_int  <= 1'b0;
    end
  else
    begin
      DQS_enable_int <= DQS_enable_out;
      DQS_reset_int  <= DQS_reset_out;
    end
end

always @ (posedge clk)
begin
 if (rst0_r == 1'b1)
   begin
     DQS_enable1     <= 1'b0;
     DQS_enable2     <= 1'b0;
     DQS_enable3     <= 1'b0;
     DQS_enable4     <= 1'b0; 
     DQS_reset1_clk0 <= 1'b0;
     DQS_reset2_clk0 <= 1'b0;
     DQS_reset3_clk0 <= 1'b0;
     DQS_reset4_clk0 <= 1'b0;
   end
 else
   begin                 
     DQS_enable1     <= DQS_enable_int;
     DQS_enable2     <= DQS_enable1;
     DQS_enable3     <= DQS_enable2;
     DQS_enable4     <= DQS_enable3;
     DQS_reset1_clk0 <= DQS_reset_int;
     DQS_reset2_clk0 <= DQS_reset1_clk0;
     DQS_reset3_clk0 <= DQS_reset2_clk0;
     DQS_reset4_clk0 <= DQS_reset3_clk0;
   end
end
                       
//****************************************************************************************************
//Generating WRITE and READ enable signals
//*****************************************************************************************************

assign write_enable_out = ((wrburst_chk == 1'b1) || (wrburst_end_cnt != 3'b000)) ? 1'b1 : 1'b0; 

//assign read_enable_out = ((CAS_COUNT != 2'b00) || (rdburst_chk == 1'b1)) ? 1'b1 : 1'b0; 
  assign read_enable_out = ((CAS_COUNT != 3'b000) || (rdburst_chk == 1'b1)) ? 1'b1 : 1'b0; //BL=8
                   
assign read_enable_out_r = read_enable_out || rdburst_end_r;

                        
/*always @ (negedge clk)       //(posedge clk180)
begin
  if (rst180_r == 1'b1)
   write_enable <= 1'b0;
  else
   write_enable <= write_enable_out;
end
*/



always @(negedge clk)    //(posedge clk180)
begin
  if (rst180_r == 1'b1)
   write_enable <= 1'b0;
  else
   write_enable <= write_enable_out;
end


/*// For Registered Dimms
always @ (negedge clk)       //(posedge clk180)   
begin
  if (rst180_r == 1'b1)
  begin
   write_enable     <= 1'b0;
   write_enable_int <= 1'b0;
 end
  else
   begin
   write_enable_int <= write_enable_out;
   write_enable     <= write_enable_int;
 end
end
*/

assign cmd_ack = ack_reg;

FD ACK_REG_INST1 (  
                  .Q(ack_reg),
                  .D(ack_o),                             
                  .C(~clk) //.C(clk180)
                  );                            

//assign ack_o = ((write_cmd2 == 1'b1) || (read_cmd3 == 1'b1)) ? 1'b1 : 1'b0;  // (write_cmd3 == 1'b1) || (read_cmd3 == 1'b1) 

//assign ack_o = ((write_cmd_in == 1'b1) || (write_cmd1 == 1'b1) || (read_cmd1 == 1'b1)) ? 1'b1 : 1'b0; 


assign ack_o = ((write_cmd_in == 1'b1) || (write_cmd1 == 1'b1) || (read_cmd1 == 1'b1)); //Sarala, June22

assign read_write_state = ((next_state1 == FIRST_WRITE) || (next_state1 == BURST_WRITE) || (next_state1 == BURST_READ)) ? 1'b1 : 1'b0;

//*********************************************************************************************
//  to initialize memory
//*********************************************************************************************
always @ (negedge clk)       //(posedge clk180)
begin
  if (rst180_r == 1'b1)
    begin
     init_memory <= 1'b0;
     INIT_DONE   <= 1'b0;
    end
  else
    begin
     init_memory <= init_mem;
     INIT_DONE   <= init_done_value && ~INIT_DONE || INIT_DONE;
    end
end

assign init_mem = (initialize_memory == 1'b1) ? 1'b1 :
                  //((INIT_COUNT == 3'b111) && (MRD_COUNT == 2'b00)) ? 1'b0 :

                   ((INIT_COUNT == 3'b111) && (mrdCnt0 == 1'b1)) ? 1'b0 :
                   init_memory;

// counter for Memory Initialization sequence

//assign INIT_COUNT_value = (( next_state1 == PRECHARGE || next_state1 == LOAD_MODE_REG || next_state1 == AUTO_REFRESH || 
//                           precharge_flag || ldMdReg_flag || aref_flag)&& init_memory == 1'b1) ? (INIT_COUNT + 1'b1) :
//                          INIT_COUNT;

assign INIT_COUNT_value = ((precharge_flag || ldMdReg_flag || aref_flag)) ? (INIT_COUNT + 1'b1) :
                          INIT_COUNT;
                    
assign init_done_value =  ((INIT_COUNT == 3'b111) && (DLL_RST_COUNT == 8'b0000_0001)) ? 1'b1 : 1'b0;                    

//Counter for DLL Reset complete
//assign DLL_RST_COUNT_value = ((init_memory == 1'b1) && (INIT_COUNT == 3'b010)) ? 8'b1100_1000 :      //200
assign DLL_RST_COUNT_value = (INIT_COUNT == 3'b010) ? 'd200 :      //200
                             (DLL_RST_COUNT != 8'b0000_0001)? (DLL_RST_COUNT - 8'b0000_0001):
                              8'b0000_0001;   

//Signal to go directly to ACTIVE state instead of LMR state after Write/Read cmd_in from user 
assign GO_TO_ACTIVE_value =( ((write_cmd_in == 1'b1) && (write_cmd1 != 1'b1))|| ((read_cmd == 1'b1) && (read_cmd1 != 1'b1)) )? 1'b1 : 1'b0;

// To check if there is a bank conflict after an ACTIVE command has been issued for a particular bank

assign CONFLICT_value  = ((RRD_COUNT == 2'b01) && (BA_address_conflict == 1'b1)) ? 1'b1 : 1'b0;
    
always @ (negedge clk)       //(posedge clk180)
begin
  if (rst180_r == 1'b1)
   begin
     CONFLICT   <= 1'b0;
     GO_TO_ACTIVE <= 1'b0;
   end
  else
   begin
     CONFLICT   <= CONFLICT_value;
     GO_TO_ACTIVE <= GO_TO_ACTIVE_value;
   end
end

//**********************************************************************************************
// Register counter values
//**********************************************************************************************
always @ (negedge clk)       //(posedge clk180)
begin
  if (rst180_r == 1'b1)
    begin
     INIT_COUNT     <= 3'b000;
     DLL_RST_COUNT  <= 8'b0000_0000;
     RP_COUNT       <= 3'b000;

     rpCnt0         <= 1'b1;
     rpCnt1         <= 1'b0;

     mrdCnt0          <= 1'b1;
     mrdCnt1          <= 1'b0;

     rcdrCnt0        <= 1'b1;
     rcdrCnt1        <= 1'b0;
     
     rcdwCnt0       <= 1'b1;
     rcdwCnt1       <= 1'b0;
     rcCnt0         <= 1'b1;

     MRD_COUNT      <= 2'b00;

     
     RFC_COUNT      <= 5'b00000;
     RAS_COUNT      <= 4'b0000;
     CAS_COUNT      <= 3'b000;  //BL = 8
     RRD_COUNT      <= 2'b00;
     RCDR_COUNT     <= 3'b000;
     RCDW_COUNT     <= 2'b00;
     RC_COUNT       <= 4'b0000;
     RDBURST_END_CNT <= 3'b000;
     wrburst_end_cnt <= 3'b000;  
     WR_COUNT        <= 2'b00;
     RFC_COUNT_reg   <= 1'b0;
     AR_Done_reg     <= 1'b0;
    end
  else
    begin
    //INIT_COUNT     <=  INIT_COUNT_value;
     INIT_COUNT     <= init_memory  ? INIT_COUNT_value : INIT_COUNT;
     DLL_RST_COUNT  <= DLL_RST_COUNT_value;
     RP_COUNT       <= rp_cnt_value;
     MRD_COUNT      <= MRD_COUNT_value;
     RFC_COUNT      <= RFC_COUNT_value;
     RAS_COUNT      <= ras_count_value;
     CAS_COUNT      <= cas_count_value;
     RRD_COUNT      <= RRD_COUNT_value;
     RCDR_COUNT     <= RCDR_COUNT_value;
     RCDW_COUNT     <= RCDW_COUNT_value;
     RC_COUNT       <= RC_COUNT_value;
     wrburst_end_cnt <= wrburst_end_cnt_value;
     RDBURST_END_CNT <= rdburst_end_cnt_value;
     WR_COUNT        <= WR_COUNT_value;

     mrdCnt0        <= (MRD_COUNT[1]  == 1'b0  &&  MRD_COUNT_value[1:0] != 2'b11);
     mrdCnt1        <= (MRD_COUNT == 2'b10);

     rpCnt0         <= (RP_COUNT[2] == 1'b0 &&  RP_COUNT[1] == 1'b0 &&  !rp_cnt_value[2]);

     rpCnt1         <= (RP_COUNT == 3'b010);


     rcdrCnt0        <= (RCDR_COUNT[2] == 1'b0 && RCDR_COUNT[1] == 1'b0 &&  !RCDR_COUNT_value[2]);
     
     rcdrCnt1        <= (RCDR_COUNT == 3'b010);


     rcdwCnt0        <= (RCDW_COUNT[1] == 1'b0 && RCDW_COUNT_value[1:0]  != 2'b11 );

     rcdwCnt1        <= (RCDW_COUNT == 2'b10);

    rcCnt0           <= (RC_COUNT[3] == 1'b0 && RC_COUNT[2] == 1'b0  && RC_COUNT[1] == 1'b0 && RC_COUNT_value[3:2] != 2'b11);
 
     if(RFC_COUNT == 5'b00010 ) //2
        AR_Done_reg <= 1'b1;
     else
       AR_Done_reg <= 1'b0;
       
     if(AR_Done_reg == 1'b1) 
        RFC_COUNT_reg <= 1'b1;
     else if (Auto_Ref_issued_p == 1'b1) 
        RFC_COUNT_reg <= 1'b0;
     else        
       RFC_COUNT_reg <= RFC_COUNT_reg;

    end
end

//*********************************************************************************************
// to check current state for the address bus
//*********************************************************************************************
always @ (negedge clk)       //(posedge clk180)
begin
  if (rst180_r == 1'b1)
   begin
      next_state    <= IDLE;
   end
  else
   begin
      if (precharge_flag)
      begin
         next_state <= PRECHARGE;
      end
      else if (ldMdReg_flag)
      begin
          next_state <= LOAD_MODE_REG; 
      end
      else if (aref_flag)
      begin
         next_state <= AUTO_REFRESH;
      end
      else if (idle_flag)
      begin
         next_state <= IDLE;
      end
      else
      begin
         next_state    <= next_state1;
      end
   end
end

//*********************************************************************************************
// main state machine
//*********************************************************************************************
always @ (next_state or rst180_r or init_memory or RP_COUNT or INIT_COUNT or MRD_COUNT or RFC_COUNT or PRECHARGE_CMD
          //or CONFLICT or auto_refresh_cmd or RCDW_COUNT or write_cmd or RCDR_COUNT or read_rcd_end or burst_length or
          or CONFLICT or RCDW_COUNT or write_cmd or RCDR_COUNT or read_rcd_end or burst_length or
          wrburst_end or wrburst_end_cnt or rdburst_end or CAS_COUNT  or WR_COUNT or GO_TO_ACTIVE or ld_mode or
          auto_ref or RFC_COUNT_reg or write_cmd3 or read_cmd5 or rpCnt0 or rpCnt1 or mrdCnt0 or mrdCnt1 
		or rcdrCnt1 or rcdwCnt1)

begin
if (rst180_r == 1'b1)
begin
	next_state1 <= IDLE;
	precharge_flag <= 1'b0;
	ldMdReg_flag <= 1'b0;
	aref_flag <= 1'b0;
	idle_flag <= 1'b0;
 end

else
  begin
  	next_state1 <= IDLE;
	precharge_flag <= 1'b0;
	ldMdReg_flag <= 1'b0;
	aref_flag <= 1'b0;
	idle_flag <= 1'b0;

    case (next_state)
        IDLE : begin
               if (init_memory == 1'b1)
                 begin
                  case (INIT_COUNT)
                   // this state is for NOP/Deselect
                   3'b000 :
                   begin
                          // next_state1 <= PRECHARGE;
                         precharge_flag <= 1'b1;
                         
                   end
                   3'b001 : 
                         begin
                           if (rpCnt1 == 1'b1)
                           begin
                           //next_state1 <= LOAD_MODE_REG;

                          
                           ldMdReg_flag <= 1'b1;
                          
                           end            
                         else

                           begin
                           //next_state1 <= IDLE;

                          idle_flag <= 1'b1;
                          end
                          
                         end
                   3'b010 : 
                         begin
                            if (mrdCnt1 == 1'b1)

                          begin
                           // next_state1 <= LOAD_MODE_REG;  // for reseting DLL in Base Mode register
                         
			
                         ldMdReg_flag <= 1'b1;
                         
                         end
            
                         else

                         begin
                          //  next_state1 <= IDLE;
                          
                            idle_flag <= 1'b1;
                           end
                         end
                   3'b011 :
                         begin
                          if (mrdCnt1 == 1'b1)
    
                          begin
                          //  next_state1 <= PRECHARGE;
                         
                           precharge_flag <= 1'b1;
                          
                          end
                          
                         else

                         begin
                           // next_state1 <= IDLE;
                         
                          idle_flag <= 1'b1;
                         end
 
                         end
                   3'b100 :
                         begin
                           if (rpCnt1 == 1'b1)

                          begin
                           // next_state1 <= AUTO_REFRESH;
                        
                         aref_flag <= 1'b1;
                        
                         end
 
                         else

                         begin
                          //  next_state1 <= IDLE;

                         idle_flag <= 1'b1;
                          end
                           
                         end
                   3'b101 :
                         begin
                         if (RFC_COUNT_reg == 1'b1)

                         begin
                          // next_state1 <= AUTO_REFRESH;
                        
                   
                          aref_flag <= 1'b1;
                         

                        end
                      
                         else 
                         begin
                          // next_state1 <= IDLE;

                          
                           idle_flag <= 1'b1;

                           end
                         end
                   3'b110 :
                         begin
                         if (RFC_COUNT_reg == 1'b1)
       
                          begin
                          //  next_state1 <= LOAD_MODE_REG;  // to deactivate the rst DLL bit in the LMR

                    
                          ldMdReg_flag <= 1'b1;
                      
                         end
                      
                         else

                           begin
                           // next_state1 <= IDLE;

                          idle_flag <= 1'b1;

                           end
                         end
                   3'b111 :
                         begin
                           if (mrdCnt0 != 1'b1)

                            begin
                           	//next_state1 <= IDLE;

                           idle_flag <= 1'b1;

                            end
	  	         else
   			    next_state1 <=next_state ;
                         end
                   default :
                       // next_state1 <= IDLE;

                       idle_flag <= 1'b1;
                   endcase 
                 end
                 else if ( auto_ref == 1'b1 && RFC_COUNT_reg == 1'b1 && rpCnt0 == 1'b1)
                  // next_state1 <= AUTO_REFRESH; // normal Refresh in the IDLE state
	          aref_flag <= 1'b1;
	   
               else if (PRECHARGE_CMD == 1'b1)
                 // next_state1 <= PRECHARGE;

		 precharge_flag <= 1'b1;
               else if (ld_mode == 1'b1)
		 ldMdReg_flag <= 1'b1;
	       else if (GO_TO_ACTIVE == 1'b1 || CONFLICT == 1'b1)
                  next_state1 <= ACTIVE; 
               else
                 // next_state1 <= IDLE;

                 idle_flag <= 1'b1;
               end

         PRECHARGE :
            // next_state1 <= IDLE;

	    idle_flag <= 1'b1;

            
         LOAD_MODE_REG :
             // next_state1 <= IDLE;

	     idle_flag <= 1'b1;
      
         AUTO_REFRESH :
            //  next_state1 <= IDLE; 

	    idle_flag <= 1'b1;
         
         ACTIVE :
              next_state1 <= ACTIVE_WAIT;
            
         ACTIVE_WAIT :
              begin  
               // if ((RCDW_COUNT == 2'b01) && (write_cmd3 == 1'b1))

                 if ((rcdwCnt1 == 1'b1) && (write_cmd3 == 1'b1))
                    next_state1 <= FIRST_WRITE;
               // else if ((RCDR_COUNT == 3'b001) && (read_cmd5 == 1'b1))

                  else if ((rcdrCnt1 == 1'b1) && (read_cmd5 == 1'b1))
                    next_state1 <= BURST_READ;
                else    
                    next_state1 <= ACTIVE_WAIT;
              end      
         
         FIRST_WRITE :
              begin
              // to meet the write preamble   
            //  if (burst_length == 3'b001)
            //      next_state1 <= BURST_WRITE;
            //  else
			if ((burst_length == 3'b001) && (wrburst_end == 1'b0))   //Added for BL = 2
                        next_state1 <= BURST_WRITE;
             else if ((burst_length == 3'b001) && (wrburst_end == 1'b1))
				next_state1 <= PRECHARGE_AFTER_WRITE;  
               else
                  next_state1 <= WRITE_WAIT;
              end
    
         WRITE_WAIT :
                     begin
                     case(wrburst_end)
                       1'b1 :
                         next_state1 <= PRECHARGE_AFTER_WRITE;  
                       1'b0 :
                         begin
                            //for RL 
                            if (wrburst_end_cnt == 3'b001)  // (wrburst_end_cnt == 3'b001)  
                                 next_state1 <= BURST_WRITE;
                            else
                                 next_state1 <= WRITE_WAIT;
                         end
                       default :
                         next_state1 <= WRITE_WAIT;
                     endcase
                     end
          BURST_WRITE :
                       begin
                  //     if (burst_length == 3'b001)
                  //        next_state1 <= BURST_WRITE;
                  //     else
              if ((burst_length == 3'b001) && (wrburst_end == 1'b0))   //Added for BL =2
                        next_state1 <= BURST_WRITE;
             else if ((burst_length == 3'b001) && (wrburst_end == 1'b1))
				next_state1 <= PRECHARGE_AFTER_WRITE;  
               else
                         next_state1 <= WRITE_WAIT;
                       end
          READ_AFTER_WRITE :
                     next_state1 <= BURST_READ;
          PRECHARGE_AFTER_WRITE :
                     next_state1 <= PRECHARGE_AFTER_WRITE_2;
          PRECHARGE_AFTER_WRITE_2 : 
          		begin  
                     if(WR_COUNT == 2'd0)   
                       //next_state1 <= PRECHARGE;
		       precharge_flag <= 1'b1;
		     else
                     next_state1 <= PRECHARGE_AFTER_WRITE_2;
                     end   
                    

          READ_WAIT : begin
                     case(rdburst_end)
                     1'b1 :
                      next_state1 <= PRECHARGE_AFTER_WRITE;
                     1'b0 :
                         begin
//                         if (CAS_COUNT == 2'b01)
                           if (CAS_COUNT == 3'b001) //For BL = 8
                             next_state1 <= BURST_READ;
                         else
                             next_state1 <= READ_WAIT;
                         end
                     default :
                         next_state1 <= READ_WAIT;
                    endcase
                     end
          BURST_READ :
                     begin
                //     if (burst_length == 3'b001)
                //          next_state1 <= BURST_READ;
                //     else
              if ((burst_length == 3'b001) && (rdburst_end == 1'b0))    //Added for BL =2
                        next_state1 <= BURST_READ;
             else if ((burst_length == 3'b001) && (rdburst_end == 1'b1))
				next_state1 <= PRECHARGE_AFTER_WRITE;  
               else
                         next_state1 <= READ_WAIT;
                     end
          BURST_STOP :
                    next_state1 <= READ_WAIT;
          default :
                next_state1 <= IDLE;
    endcase
  end
end

//************************************************************************************************
// address generation logic
//************************************************************************************************

assign address_config[`row_address-1:7] = (INIT_COUNT == 3'b010) ? {`row_address-7{1'b0}} :
                                          (INIT_COUNT == 3'b011) ? {{`row_address-9{1'b0}},2'b10}:
                                          (next_state == PRECHARGE) ? 8 :
                                          {`row_address-7{1'b0}};

assign address_config[6:4] =  (INIT_COUNT == 3'b010) ? 3'b000 :
                              (next_state == LOAD_MODE_REG) ? cas_latency :
                              3'b000;
                              
assign address_config[3] = 1'b0; // design uses sequential burst

assign address_config[2:0] =  (INIT_COUNT == 3'b010) ? 3'b000 :
                              (next_state == LOAD_MODE_REG) ? burst_length :
                              3'b000;


assign ddr_address1 = (next_state == LOAD_MODE_REG || next_state == PRECHARGE) ? address_config :
                      (next_state == ACTIVE) ? row_address_reg :
//                      ((next_state == BURST_WRITE) || (next_state == FIRST_WRITE) || (next_state == BURST_READ)) ? {{`row_address - `column_address{1'b0}},column_address_reg} : 
                        ((next_state == BURST_WRITE) || (next_state == FIRST_WRITE) || (next_state == BURST_READ)) ? column_address_reg : 
                      `row_address'b0;
                      
assign ddr_ba1 =  ((next_state == LOAD_MODE_REG) && (INIT_COUNT == 3'b010)) ? {{`bank_address-1{1'b0}},1'b1} :
                  ((next_state == ACTIVE) || (next_state == FIRST_WRITE) || (next_state == BURST_WRITE) || (next_state == BURST_READ)) ? BA_address_reg :
                  {{`bank_address-1{1'b0}},1'b0};



//********************************************************************************************************
//  register row address
//********************************************************************************************************
always @ (negedge clk)       //(posedge clk180)
begin
  if (rst180_r == 1'b1)
     row_address_active_reg <= `row_address'b0;
  else
   begin
     if (next_state == ACTIVE)
         row_address_active_reg <= row_address_reg;
     else
         row_address_active_reg <= row_address_active_reg;
   end
end

always @ (negedge clk)       //(posedge clk180)
begin
  if (rst180_r == 1'b1)
     row_address_conflict <= 1'b0;
  else
   begin
     if (row_address_reg != row_address_active_reg)
         row_address_conflict <= 1'b1;
     else
         row_address_conflict <= 1'b0;
   end
end

//********************************************************************************************************
//  register bank address
//********************************************************************************************************

always @ (negedge clk)       //(posedge clk180)
begin
  if (rst180_r == 1'b1)
      BA_address_active <= `bank_address'b0;
  else
    begin
      if (next_state == ACTIVE)
          BA_address_active <= BA_address_reg;
      else
          BA_address_active <= BA_address_active;
    end
end

always @ (negedge clk)       //(posedge clk180)
begin
  if (rst180_r == 1'b1)
      BA_address_conflict <= 1'b0;
  else
    begin
      if (BA_address_reg != BA_address_active)
          BA_address_conflict <= 1'b1;
      else
          BA_address_conflict <= 1'b0;
    end
end

//********************************************************************************************************
//  register column address
//********************************************************************************************************
always @ (negedge clk)       //(posedge clk180)
begin
  if (rst180_r == 1'b1)
    begin
 /*    column_address_reg1 <= `column_address'b0;
     column_address_reg2 <= `column_address'b0;
     column_address_reg3 <= `column_address'b0;
     column_address_reg4 <= `column_address'b0;
     column_address_reg5 <= `column_address'b0;
     column_address_reg6 <= `column_address'b0; 
 */
     column_address_reg1 <= `row_address'b0;
     column_address_reg2 <= `row_address'b0;
     column_address_reg3 <= `row_address'b0;
     column_address_reg4 <= `row_address'b0;
     column_address_reg5 <= `row_address'b0;
     column_address_reg6 <= `row_address'b0; 

    end
  else
    begin
     column_address_reg1 <= column_address_reg;
     column_address_reg2 <= column_address_reg1;
     column_address_reg3 <= column_address_reg2;
     column_address_reg4 <= column_address_reg3;
     column_address_reg5 <= column_address_reg4;
     column_address_reg6 <= column_address_reg5;
    end
end



//**************************************************************************************************
//Pipeline stages for ddr_address and ddr_ba
//**************************************************************************************************

always @ (negedge clk)       //(posedge clk180)
begin
if (rst180_r == 1'b1)
  begin
   ddr_address2  <= `row_address'b0;
   ddr_address3  <= `row_address'b0;
   ddr_ba2       <= `bank_address'b0;
   ddr_ba3       <= `bank_address'b0;
  end
else
  begin
    ddr_address2 <= ddr_address1;
    ddr_address3 <= ddr_address2;
    ddr_ba2      <= ddr_ba1;
    ddr_ba3      <= ddr_ba2;
  end
end

always @ (negedge clk)       //(posedge clk180)
begin
  if (rst180_r == 1'b1)
    begin
      ddr_address4  <= `row_address'b0;
      ddr_address5  <= `row_address'b0;
      ddr_address_cntrl   <= `row_address'b0;
    end
  else
    begin
      ddr_address4  <= ddr_address3;
      ddr_address5  <= ddr_address4;
     ddr_address_cntrl   <= ddr_address1;
    end
end

always @ (negedge clk)       //(posedge clk180)
begin
  if (rst180_r == 1'b1)
    begin
     ddr_ba4 <= `bank_address'b0;
     ddr_ba5 <= `bank_address'b0;
     ddr_ba_cntrl  <= `bank_address'b0;
    end
  else
    begin
     ddr_ba4 <= ddr_ba3;
     ddr_ba5 <= ddr_ba4;
     ddr_ba_cntrl  <= ddr_ba1;

    end
end

//************************************************************************************************
// control signals to the Memory
//************************************************************************************************

assign ddr_rasb1 = ((next_state == ACTIVE) || (next_state == PRECHARGE) || (next_state == AUTO_REFRESH) || (next_state == LOAD_MODE_REG)) ? 1'b0 : 1'b1;

assign ddr_casb1 = ((next_state == BURST_READ) || (next_state == BURST_WRITE) || (next_state == FIRST_WRITE) || 
                   (next_state == AUTO_REFRESH) || (next_state == LOAD_MODE_REG)) ? 1'b0 : 1'b1;

assign ddr_web1  = ((next_state == BURST_WRITE) || (next_state == FIRST_WRITE) || (next_state == BURST_STOP) || 
                   (next_state == PRECHARGE) || (next_state == LOAD_MODE_REG)) ? 1'b0 : 1'b1;


//*************************************************************************************************
// register CONTROL SIGNALS outputs
//**************************************************************************************************
always @ (negedge clk)       //(posedge clk180)
begin
  if (rst180_r == 1'b1)
    begin
      ddr_rasb3 <= 1'b1;
      ddr_casb3 <= 1'b1;
      ddr_web3  <= 1'b1;
      ddr_rasb2 <= 1'b1;
      ddr_casb2 <= 1'b1;
      ddr_web2  <= 1'b1;
    end
  else
    begin
      ddr_rasb2    <= ddr_rasb1;
      ddr_casb2    <= ddr_casb1;
      ddr_web2     <= ddr_web1;
      ddr_rasb3     <= ddr_rasb2;
      ddr_casb3    <= ddr_casb2;
      ddr_web3     <= ddr_web2;
    end
end

always @ (negedge clk)       //(posedge clk180)
begin
  if (rst180_r == 1'b1)
    begin
      ddr_rasb4  <= 1'b1;
      ddr_casb4  <= 1'b1;
      ddr_web4   <= 1'b1; 
      ddr_rst_dqs_rasb4  <= 1'b1;
      ddr_rst_dqs_casb4  <= 1'b1;
      ddr_rst_dqs_web4   <= 1'b1; 
      ddr_rasb5 <= 1'b1;
      ddr_casb5 <= 1'b1;
      ddr_web5  <= 1'b1;
    end
  else
    begin
      ddr_rasb4  <= ddr_rasb3;
      ddr_casb4  <= ddr_casb3;
      ddr_web4   <= ddr_web3;
      ddr_rasb5   <= ddr_rasb4;
      ddr_casb5   <= ddr_casb4;
      ddr_web5    <= ddr_web4;
    
	 if(cas_latency == 3'b011) // --CL3
	 begin
		ddr_rst_dqs_rasb4  <= ddr_rasb1;
   		ddr_rst_dqs_casb4  <= ddr_casb1;
   		ddr_rst_dqs_web4   <= ddr_web1; 
	 end
	 
	else
	begin
	  ddr_rst_dqs_rasb4  <= ddr_rst_dqs_rasb4;
   	ddr_rst_dqs_casb4  <= ddr_rst_dqs_casb4;
   	ddr_rst_dqs_web4   <= ddr_rst_dqs_web4; 
	end
	end
end


/*
always @ (negedge clk)       //(posedge clk180)
begin
    if (rst180_r == 1'b1)
        dqs_div_cascount <= 3'b000;
    else
      begin
  //      if( ((ddr_rasb1 == 1'b1) && (ddr_casb1 == 1'b0) && (ddr_web1 == 1'b1) && (burst_length == 3'b010) && ((cas_latency == 3'b010)||(cas_latency == 3'b110)) )  || 
//         if( ((ddr_rasb1 == 1'b1) && (ddr_casb1 == 1'b0) && (ddr_web1 == 1'b1) && (burst_length == 3'b011) && ((cas_latency == 3'b010)||(cas_latency == 3'b110)) )  ||  //BL=8
  //         ((ddr_rst_dqs_rasb4 == 1'b1) && (ddr_rst_dqs_casb4 == 1'b0) && (ddr_rst_dqs_web4 == 1'b1) && (burst_length == 3'b010) && (cas_latency == 3'b011) ) )
           //  dqs_div_cascount <= 3'b010;
//		  dqs_div_cascount <= burst_cnt_max ; // Modified by Abhishake for BL=8 


  if( (ddr_rasb1 == 1'b1) && (ddr_casb1 == 1'b0) && (ddr_web1 == 1'b1) && ((cas_latency == 3'b010)||(cas_latency == 3'b110)) )  // For CL= 2,2.5 & BL=8,4
          dqs_div_cascount <= burst_cnt_max ; // Modified by Abhishake for BL=8

//  else if(((ddr_rasb1 == 1'b1) && (ddr_casb1 == 1'b0) && (ddr_web1 == 1'b1) && (burst_length == 3'b011) && (cas_latency == 3'b011) )  ||
//           ( (ddr_rst_dqs_rasb4 == 1'b1) && (ddr_rst_dqs_casb4 == 1'b0) && (ddr_rst_dqs_web4 == 1'b1) && (burst_length == 3'b011) && (cas_latency == 3'b011) )  )
//             dqs_div_cascount <= 3'b100; 


   else if((ddr_rst_dqs_rasb4 == 1'b1) && (ddr_rst_dqs_casb4 == 1'b0) && (ddr_rst_dqs_web4 == 1'b1)) // For BL = 4,8 & CL=3
             dqs_div_cascount <= burst_cnt_max ;

        else
          begin
             if (dqs_div_cascount != 3'b000)
                 dqs_div_cascount <= dqs_div_cascount - 1'b1;
             else
                 dqs_div_cascount <= dqs_div_cascount;
          end
      end
end
*/

always @ (negedge clk)       //(posedge clk180)
begin
    if (rst180_r == 1'b1)
        dqs_div_cascount <= 3'b000;
    else
      begin
  if( (ddr_rasb1 == 1'b1) && (ddr_casb1 == 1'b0) && (ddr_web1 == 1'b1) && ((cas_latency == 3'b010)||(cas_latency == 3'b110)) )  // For CL= 2,2.5 & BL=8,4
          dqs_div_cascount <= burst_cnt_max ; // Modified by Abhishake for BL=8

   else if((ddr_rst_dqs_rasb4 == 1'b1) && (ddr_rst_dqs_casb4 == 1'b0) && (ddr_rst_dqs_web4 == 1'b1)) // For BL = 4,8 & CL=3
             dqs_div_cascount <= burst_cnt_max ;
   else
    begin
     if (dqs_div_cascount != 3'b000)
        dqs_div_cascount <= dqs_div_cascount - 1'b1;
      else
       dqs_div_cascount <= dqs_div_cascount;
       end
  end
end

always @ (negedge clk)       //(posedge clk180)
begin
    if (rst180_r == 1'b1)
        dqs_div_rdburstcount <= 3'b000;
    else
      begin
             if (dqs_div_cascount == 3'b001  && burst_length == 3'b010)
            dqs_div_rdburstcount <= 3'b010; 
        else if (dqs_div_cascount == 3'b011  && burst_length == 3'b011)
            dqs_div_rdburstcount <= 3'b100; 
        else if (dqs_div_cascount == 3'b001  && burst_length == 3'b001) // Added for BL = 2
            dqs_div_rdburstcount <= 3'b001;                             // Added for BL = 2  

        else 
          begin
            if (dqs_div_rdburstcount != 3'b000)
               dqs_div_rdburstcount <= dqs_div_rdburstcount - 1'b1;
            else
               dqs_div_rdburstcount <= dqs_div_rdburstcount;
          end
      end
end


always @ (negedge clk)       //(posedge clk180)
begin
    if (rst180_r == 1'b1)
        rst_dqs_div_r <= 1'b0;
    else
      begin
        if (dqs_div_cascount == 3'b001  && burst_length == 3'b010)
            rst_dqs_div_r <= 1'b1;
        else if (dqs_div_cascount == 3'b011  && burst_length == 3'b011)
            rst_dqs_div_r <= 1'b1;
	  else if (dqs_div_cascount == 3'b001  && burst_length == 3'b001)   // Added for BL = 2
            rst_dqs_div_r <= 1'b1;                                         // Added for BL = 2 

        else if (dqs_div_rdburstcount == 3'b001 && dqs_div_cascount == 3'b000)
            rst_dqs_div_r <= 1'b0;
        else
            rst_dqs_div_r <= rst_dqs_div_r;
      end
end

/*//The Signal dly_dqs_div_r is used only for BL = 2 to delay the rst_dqs_div_r by 1 clock pulse
always @ (negedge clk)       //(posedge clk180)
begin
    if (rst180_r == 1'b1)
        dly_dqs_div_r <= 1'b0;
    else
        dly_dqs_div_r <= rst_dqs_div_r ;
 end
   
always @(negedge clk)       //(posedge clk180)    //For BL=2,4,8 n Reg Dimm
begin
   if (rst180_r == 1'b1)
     begin
      dly_dqs_div_r   &lt;= 1'b0;
      rst_dqs_div_r11 &lt;= 1'b0;
      rst_dqs_div_r1  &lt;= 1'b0;
     end
   else if (burst_length == 3'b001)
     begin 
      rst_dqs_div_r11 &lt;= rst_dqs_div_r;
      dly_dqs_div_r   &lt;= rst_dqs_div_r11;
     end
   else
      rst_dqs_div_r1  &lt;= rst_dqs_div_r; 
end
*/


//For BL=2,4,8 n Comp or Unbuffered Dimms
assign rst_dqs_div_r1 = rst_dqs_div_r;

always @(negedge clk)    //(posedge clk180)
begin
  if (rst180_r == 1'b1)
    dly_dqs_div_r <= 1'b0;
  else
    dly_dqs_div_r <= rst_dqs_div_r;
end


/*
//For BL=2,4,8 n Reg Dimm
assign rst_dqs_div_r1 <= rst_dqs_div_r;

always @(negedge clk)       //(posedge clk180)
begin
  if (rst180_r == 1'b1)
    dly_dqs_div_r <= 1'b0;
  else
    dly_dqs_div_r <= rst_dqs_div_r;
end
*/
        
/* FD  rst_calib0  (
                 .Q(rst_calib),     
                 .D(dly_dqs_div_r),  //BL = 2
              // .D(rst_dqs_div_r),  //BL = 4,8                                              
                 .C(clk180)
                 );         
*/

 FD  rst_calib0  (
                 .Q(rst_calib),     
                 
   .D(dly_dqs_div_r),  

                 
		                                              
                 .C(~clk)     //.C(clk180)
                 );         
        


/*
FD  rst_iob_out (
                 .Q(rst_dqs_div_int),     
                 .D(dly_dqs_div_r),   //BL = 2
		//   .D(rst_dqs_div_r1),  // BL = 4,8                       
//               .C(clk)           //For CL = 2 and 3
                 .C(clk180)       //For CL= 2.5
                 );
*/  

FD  rst_iob_out (
                 .Q(rst_dqs_div_int),     
                 
   .D(dly_dqs_div_r),    

       	     
	           
		     
		     
   .C(clk)  

       	     
	           
		     	
                 );
   
                 
endmodule   
