//DDR SDRAM common interface 
//Frequency divider by 1.5

// Company: Quenne Electric
// Engineer: Andras Bognar

`timescale 1ps / 1ps

module freqdivider(CLKINPUT, CLKOUTPUT);
    input CLKINPUT;
    output CLKOUTPUT;
	
	wire CLKDV_BUF;
	wire CLKIN_IBUFG;
	wire CLKFB_IN;
	wire CLK0_BUF;
	
	BUFG CLKDV_BUFG_INST (.I(CLKDV_BUF),.O(CLKOUTPUT));
	IBUFG CLKIN_IBUFG_INST (.I(CLKINPUT),.O(CLKIN_IBUFG));
	BUFG CLK0_BUFG_INST (.I(CLK0_BUF),.O(CLKFB_IN));

	DCM_SP  #(
      .CLKDV_DIVIDE(1.5), // Divide by: 1.5,2.0,2.5,3.0,3.5,4.0,4.5,5.0,5.5,6.0,6.5
                          //   7.0,7.5,8.0,9.0,10.0,11.0,12.0,13.0,14.0,15.0 or 16.0
      .CLKFX_DIVIDE(25),   // Can be any integer from 1 to 32
      .CLKFX_MULTIPLY(4), // Can be any integer from 2 to 32
      .CLKIN_DIVIDE_BY_2("FALSE"), // TRUE/FALSE to enable CLKIN divide by two feature
      .CLKIN_PERIOD(5.0),  // Specify period of input clock
      .CLKOUT_PHASE_SHIFT("NONE"), // Specify phase shift of NONE, FIXED or VARIABLE
      .CLK_FEEDBACK("1X"),  // Specify clock feedback of NONE, 1X or 2X
      .DESKEW_ADJUST("SYSTEM_SYNCHRONOUS"), // SOURCE_SYNCHRONOUS, SYSTEM_SYNCHRONOUS or
                                            //   an integer from 0 to 15
      .DFS_FREQUENCY_MODE("LOW"),  // HIGH or LOW frequency mode for frequency synthesis
      .DLL_FREQUENCY_MODE("LOW"),  // HIGH or LOW frequency mode for DLL
      .DUTY_CYCLE_CORRECTION("FALSE"), // Duty cycle correction, TRUE or FALSE
      .FACTORY_JF(16'hC080),   // FACTORY JF values
      .PHASE_SHIFT(0),     // Amount of fixed phase shift from -255 to 255
      .STARTUP_WAIT("FALSE")   // Delay configuration DONE until DCM LOCK, TRUE/FALSE
	) DCM_SP_inst (
	  .CLK0(CLK0_BUF),     // 0 degree DCM CLK output
      //.CLK180(CLK180), // 180 degree DCM CLK output
      //.CLK270(CLK270), // 270 degree DCM CLK output
      //.CLK2X(CLKOUTPUT),   // 2X DCM CLK output
      //.CLK2X180(CLK2X180), // 2X, 180 degree DCM CLK out
      //.CLK90(CLK90),   // 90 degree DCM CLK output
      .CLKDV(CLKDV_BUF),   // Divided DCM CLK out (CLKDV_DIVIDE)
      //.CLKFX(CLKOUTPUT),   // DCM CLK synthesis out (M/D)
      //.CLKFX180(CLKFX180), // 180 degree CLK synthesis out
      //.LOCKED(LOCKED), // DCM LOCK status output
      //.PSDONE(PSDONE), // Dynamic phase adjust done output
      //.STATUS(STATUS), // 8-bit DCM status bits output
      .CLKFB(CLKFB_IN),   // DCM clock feedback
      .CLKIN(CLKIN_IBUFG),   // Clock input (from IBUFG, BUFG or DCM)
	  .DSSEN(1'h0),
      .PSCLK(1'h0),   // Dynamic phase adjust clock input
      .PSEN(1'h0),     // Dynamic phase adjust enable input
      .PSINCDEC(1'h0), // Dynamic phase adjust increment/decrement
      .RST(1'h0)        // DCM asynchronous reset input
	);

endmodule
