//DDR SDRAM common interface 
//Automat to give common interface to DDR SDRAM

// Company: Quenne Electric
// Engineer: Andras Bognar

`timescale 1ps / 1ps

module ddr_top(
//200MHz clock in
	CLKIN,
//interface signals to the DDR SDRAM
	A,		//13bit address
	DQ,		//16bit data
	BA,		//bank adress
	RAS,	//row address
	CAS,	//column address
	WE,		//write enable
	UQM,	//upper data mask
	LQM,	//lower data mask
	UDQS,	//upper data strobe
	LDQS,	//lower data strobe
	CS,		//chip select
	CKE,	//clock enable
	CLK_P,	//clock ponated
	CLK_N,	//clock negated
	CLKFB,	//clock feedback
//signal LED outputs [for test report]
	LED1,
	LED2
);
	
//CLOCK
	input CLKIN;
	output CLK_P;	//clock ponated
	output CLK_N;	//clock negated
	input  CLKFB;	//clock feedback
//SDRAM IF
	output [12:0]A;
	inout [15:0]DQ;
	output [1:0]BA;
	output RAS;
	output CAS;
	output WE;
	output UQM;
	output LQM;
	inout UDQS;
	inout LDQS;
	output CS;
	output CKE;
//LED signals
	output LED1;
	output LED2;
	
//internal connections
	wire [23:0]addr;
	wire [31:0]datain;
	wire [31:0]dataout;
	wire w;
	wire ready;
	wire str;
	wire ddrclk;
	reg sysclk=0;
	
	
	//100MHz->50MHz
	always@(posedge ddrclk)
		begin
		sysclk<=sysclk?0:1;
		end

	cpuemu cpu1(	//processor emulator for test benches
		sysclk,		//50MHz
		addr,
		datain,
		dataout,
		w,
		ready,
		str,
		LED1,
		LED2
	);	
	
	ddraut ddraut1(	//ddr controller automat
	//interface signals to the DDR SDRAM
		A,		//13bit address
		DQ,		//16bit data
		BA,		//bank adress
		RAS,	//row address
		CAS,	//column address
		WE,		//write enable
		UQM,	//upper data mask
		LQM,	//lower data mask
		UDQS,	//upper data strobe
		LDQS,	//lower data strobe
		CS,		//chip select
		CKE,	//clock enable
	//clocks
		CLKIN,	//clock input 200MHz
		ddrclk,	//clock for ddr ram 100MHz
	//Interface signals to the HOST	
		addr,	//address bus
		datain,	//data bus
		dataout,//data bus
		w,		//command: write, not read
		ready,	//execution status
		str		//strobe for command		
	);	
	
	clkshifter clksh1(ddrclk,CLK_P,CLK_N,CLKFB);	//ddr ram clock gen.	

endmodule
